@@ -21,6 +21,7 @@
 #include <jni.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdint.h>
 #include <glib.h>
 #include <atk-bridge.h>
 #include <X11/Xlib.h>
@@ -47,6 +48,9 @@ extern "C" {
 #define GDK_MOD5_MASK (1 << 7)
 #define GDK_META_MASK (1 << 28)
 
+#define JAW_LOG_FILE "jaw_log.txt"
+#define JAW_LOG_FILE2 "/tmp/"JAW_LOG_FILE
+
 typedef enum _SignalType SignalType;
 gboolean jaw_accessibility_init (void);
 void jaw_accessibility_shutdown (void);
@@ -99,10 +103,16 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_initNativeLibrary(void)
   }
   if (jaw_debug)
   {
-    jaw_log_file = fopen("jaw_log_file.txt","w+");
+    jaw_log_file = fopen(JAW_LOG_FILE, "w+");
     if (!jaw_log_file)
     {
-      fprintf(stderr, "Error opening log file jaw_log_file.txt\n");
+      perror("Error opening log file "JAW_LOG_FILE", trying "JAW_LOG_FILE2);
+      jaw_log_file = fopen(JAW_LOG_FILE2, "w+");
+    }
+
+    if (!jaw_log_file)
+    {
+      perror("Error opening log file "JAW_LOG_FILE2);
       exit(1);
     }
     jaw_start_time = time(NULL);
@@ -169,6 +179,11 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_loadAtkBridge(void)
     JAW_DEBUG_I("Thread create failed: %s !", err->message);
     g_error_free (err);
   }
+  else
+  {
+    /* We won't join it */
+    g_thread_unref(thread);
+  }
 }
 
 JNIEXPORT void
@@ -204,6 +219,7 @@ enum _SignalType {
 };
 
 typedef struct _CallbackPara {
+  jobject ac;
   jobject global_ac;
   JawImpl *jaw_impl;
   JawImpl *child_impl;
@@ -628,6 +644,15 @@ get_int_value (JNIEnv *jniEnv, jobject o)
   return (gint)(*jniEnv)->CallIntMethod(jniEnv, o, jmid);
 }
 
+/*
+ * OpenJDK seems to be sending flurries of visible data changed events, which
+ * overloads us. They are however usually just for the same object, so we can
+ * compact them: there is no need to queue another one if the previous hasn't
+ * even been sent!
+ */
+static pthread_mutex_t jaw_vdc_dup_mutex = PTHREAD_MUTEX_INITIALIZER;
+static jobject jaw_vdc_last_ac = NULL;
+
 static gboolean
 signal_emit_handler (gpointer p)
 {
@@ -637,6 +662,16 @@ signal_emit_handler (gpointer p)
   jobjectArray args = para->args;
   AtkObject* atk_obj = ATK_OBJECT(para->jaw_impl);
 
+  if (para->signal_id == Sig_Object_Visible_Data_Changed)
+  {
+    pthread_mutex_lock(&jaw_vdc_dup_mutex);
+    if (jaw_vdc_last_ac == para->ac)
+      /* So we will be sending the visible data changed event. If any other
+       * comes, we will want to send it  */
+      jaw_vdc_last_ac = NULL;
+    pthread_mutex_unlock(&jaw_vdc_dup_mutex);
+  }
+
   switch (para->signal_id)
   {
     case Sig_Text_Caret_Moved:
@@ -868,14 +903,38 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_emitSignal(JNIEnv *jniEnv,
                                                            jobjectArray args)
 {
   JAW_DEBUG_JNI("%p, %p, %p, %d, %p", jniEnv, jClass, jAccContext, id, args);
+
+  pthread_mutex_lock(&jaw_vdc_dup_mutex);
+  if (id != Sig_Object_Visible_Data_Changed)
+  {
+    /* Something may have happened since the last visible data changed event, so
+     * we want to sent it again */
+    jaw_vdc_last_ac = NULL;
+  }
+  else
+  {
+    if (jaw_vdc_last_ac == jAccContext)
+    {
+      /* We have already queued to send one and nothing happened in between,
+       * this one is really useless */
+      pthread_mutex_unlock(&jaw_vdc_dup_mutex);
+      return;
+    }
+
+    jaw_vdc_last_ac = jAccContext;
+  }
+  pthread_mutex_unlock(&jaw_vdc_dup_mutex);
+
   if (!jAccContext) {
     JAW_DEBUG_I("jAccContext == NULL");
     return;
   }
+
   jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   callback_para_process_frees();
   jobjectArray global_args = (jobjectArray)(*jniEnv)->NewGlobalRef(jniEnv, args);
   CallbackPara *para = alloc_callback_para(jniEnv, global_ac);
+  para->ac = jAccContext;
   para->signal_id = (gint)id;
   para->args = global_args;
   switch (para->signal_id)
@@ -1227,7 +1286,7 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_getInstance(JNIEnv *jniEnv,
   if (!ac)
     return 0;
 
-  return (jlong) jaw_impl_get_instance(jniEnv, ac);
+  return (jlong) (uintptr_t) jaw_impl_get_instance(jniEnv, ac);
 }
 
 #ifdef __cplusplus
